@@ -119,7 +119,7 @@ class Corr:
     def gamma_method(self, **kwargs):
         """Apply the gamma method to the content of the Corr."""
         for item in self.content:
-            if not(item is None):
+            if not (item is None):
                 if self.N == 1:
                     item[0].gamma_method(**kwargs)
                 else:
@@ -139,7 +139,7 @@ class Corr:
 
         if vector_l is None:
             vector_l, vector_r = np.asarray([1.] + (self.N - 1) * [0.]), np.asarray([1.] + (self.N - 1) * [0.])
-        elif(vector_r is None):
+        elif (vector_r is None):
             vector_r = vector_l
         if isinstance(vector_l, list) and not isinstance(vector_r, list):
             if len(vector_l) != self.T:
@@ -178,7 +178,7 @@ class Corr:
         """
         if self.N == 1:
             raise Exception("Trying to pick item from projected Corr")
-        newcontent = [None if(item is None) else item[i, j] for item in self.content]
+        newcontent = [None if (item is None) else item[i, j] for item in self.content]
         return Corr(newcontent)
 
     def plottable(self):
@@ -211,7 +211,7 @@ class Corr:
                 newcontent.append(None)
             else:
                 newcontent.append(0.5 * (self.content[t] + self.content[self.T - t]))
-        if(all([x is None for x in newcontent])):
+        if (all([x is None for x in newcontent])):
             raise Exception("Corr could not be symmetrized: No redundant values")
         return Corr(newcontent, prange=self.prange)
 
@@ -233,17 +233,34 @@ class Corr:
                 newcontent.append(None)
             else:
                 newcontent.append(0.5 * (self.content[t] - self.content[self.T - t]))
-        if(all([x is None for x in newcontent])):
+        if (all([x is None for x in newcontent])):
             raise Exception("Corr could not be symmetrized: No redundant values")
         return Corr(newcontent, prange=self.prange)
 
+    def is_matrix_symmetric(self):
+        """Checks whether a correlator matrices is symmetric on every timeslice."""
+        if self.N == 1:
+            raise Exception("Only works for correlator matrices.")
+        for t in range(self.T):
+            if self[t] is None:
+                continue
+            for i in range(self.N):
+                for j in range(i + 1, self.N):
+                    if self[t][i, j] is self[t][j, i]:
+                        continue
+                    if hash(self[t][i, j]) != hash(self[t][j, i]):
+                        return False
+        return True
+
     def matrix_symmetric(self):
         """Symmetrizes the correlator matrices on every timeslice."""
-        if self.N > 1:
-            transposed = [None if _check_for_none(self, G) else G.T for G in self.content]
-            return 0.5 * (Corr(transposed) + self)
         if self.N == 1:
             raise Exception("Trying to symmetrize a correlator matrix, that already has N=1.")
+        if self.is_matrix_symmetric():
+            return 1.0 * self
+        else:
+            transposed = [None if _check_for_none(self, G) else G.T for G in self.content]
+            return 0.5 * (Corr(transposed) + self)
 
     def GEVP(self, t0, ts=None, sort="Eigenvalue", **kwargs):
         r'''Solve the generalized eigenvalue problem on the correlator matrix and returns the corresponding eigenvectors.
@@ -284,7 +301,11 @@ class Corr:
             warnings.warn("Argument 'sorted_list' is deprecated, use 'sort' instead.", DeprecationWarning)
             sort = kwargs.get("sorted_list")
 
-        symmetric_corr = self.matrix_symmetric()
+        if self.is_matrix_symmetric():
+            symmetric_corr = self
+        else:
+            symmetric_corr = self.matrix_symmetric()
+
         if sort is None:
             if (ts is None):
                 raise Exception("ts is required if sort=None.")
@@ -513,7 +534,7 @@ class Corr:
                     newcontent.append(None)
                 else:
                     newcontent.append(0.5 * (self.content[t + 1] - self.content[t - 1]))
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception('Derivative is undefined at all timeslices')
             return Corr(newcontent, padding=[1, 1])
         elif variant == "forward":
@@ -523,7 +544,7 @@ class Corr:
                     newcontent.append(None)
                 else:
                     newcontent.append(self.content[t + 1] - self.content[t])
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception("Derivative is undefined at all timeslices")
             return Corr(newcontent, padding=[0, 1])
         elif variant == "backward":
@@ -533,7 +554,7 @@ class Corr:
                     newcontent.append(None)
                 else:
                     newcontent.append(self.content[t] - self.content[t - 1])
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception("Derivative is undefined at all timeslices")
             return Corr(newcontent, padding=[1, 0])
         elif variant == "improved":
@@ -543,7 +564,7 @@ class Corr:
                     newcontent.append(None)
                 else:
                     newcontent.append((1 / 12) * (self.content[t - 2] - 8 * self.content[t - 1] + 8 * self.content[t + 1] - self.content[t + 2]))
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception('Derivative is undefined at all timeslices')
             return Corr(newcontent, padding=[2, 2])
         else:
@@ -567,7 +588,7 @@ class Corr:
                     newcontent.append(None)
                 else:
                     newcontent.append((self.content[t + 1] - 2 * self.content[t] + self.content[t - 1]))
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception("Derivative is undefined at all timeslices")
             return Corr(newcontent, padding=[1, 1])
         elif variant == "improved":
@@ -577,7 +598,7 @@ class Corr:
                     newcontent.append(None)
                 else:
                     newcontent.append((1 / 12) * (-self.content[t + 2] + 16 * self.content[t + 1] - 30 * self.content[t] + 16 * self.content[t - 1] - self.content[t - 2]))
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception("Derivative is undefined at all timeslices")
             return Corr(newcontent, padding=[2, 2])
         else:
@@ -602,11 +623,13 @@ class Corr:
         if variant == 'log':
             newcontent = []
             for t in range(self.T - 1):
-                if (self.content[t] is None) or (self.content[t + 1] is None):
+                if ((self.content[t] is None) or (self.content[t + 1] is None)) or (self.content[t + 1][0].value == 0):
+                    newcontent.append(None)
+                elif self.content[t][0].value / self.content[t + 1][0].value < 0:
                     newcontent.append(None)
                 else:
                     newcontent.append(self.content[t] / self.content[t + 1])
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception('m_eff is undefined at all timeslices')
 
             return np.log(Corr(newcontent, padding=[0, 1]))
@@ -622,14 +645,16 @@ class Corr:
 
             newcontent = []
             for t in range(self.T - 1):
-                if (self.content[t] is None) or (self.content[t + 1] is None):
+                if (self.content[t] is None) or (self.content[t + 1] is None) or (self.content[t + 1][0].value == 0):
                     newcontent.append(None)
                 # Fill the two timeslices in the middle of the lattice with their predecessors
                 elif variant == 'sinh' and t in [self.T / 2, self.T / 2 - 1]:
                     newcontent.append(newcontent[-1])
+                elif self.content[t][0].value / self.content[t + 1][0].value < 0:
+                    newcontent.append(None)
                 else:
                     newcontent.append(np.abs(find_root(self.content[t][0] / self.content[t + 1][0], root_function, guess=guess)))
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception('m_eff is undefined at all timeslices')
 
             return Corr(newcontent, padding=[0, 1])
@@ -637,11 +662,11 @@ class Corr:
         elif variant == 'arccosh':
             newcontent = []
             for t in range(1, self.T - 1):
-                if (self.content[t] is None) or (self.content[t + 1] is None) or (self.content[t - 1] is None):
+                if (self.content[t] is None) or (self.content[t + 1] is None) or (self.content[t - 1] is None) or (self.content[t][0].value == 0):
                     newcontent.append(None)
                 else:
                     newcontent.append((self.content[t + 1] + self.content[t - 1]) / (2 * self.content[t]))
-            if(all([x is None for x in newcontent])):
+            if (all([x is None for x in newcontent])):
                 raise Exception("m_eff is undefined at all timeslices")
             return np.arccosh(Corr(newcontent, padding=[1, 1]))
 
@@ -704,7 +729,7 @@ class Corr:
                 raise Exception("no plateau range provided")
         if self.N != 1:
             raise Exception("Correlator must be projected before getting a plateau.")
-        if(all([self.content[t] is None for t in range(plateau_range[0], plateau_range[1] + 1)])):
+        if (all([self.content[t] is None for t in range(plateau_range[0], plateau_range[1] + 1)])):
             raise Exception("plateau is undefined at all timeslices in plateaurange.")
         if auto_gamma:
             self.gamma_method()
@@ -731,32 +756,34 @@ class Corr:
         self.prange = prange
         return
 
-    def show(self, x_range=None, comp=None, y_range=None, logscale=False, plateau=None, fit_res=None, ylabel=None, save=None, auto_gamma=False, hide_sigma=None, references=None):
+    def show(self, x_range=None, comp=None, y_range=None, logscale=False, plateau=None, fit_res=None, ylabel=None, save=None, auto_gamma=False, hide_sigma=None, references=None, title=None):
         """Plots the correlator using the tag of the correlator as label if available.
 
         Parameters
         ----------
         x_range : list
-            list of two values, determining the range of the x-axis e.g. [4, 8]
+            list of two values, determining the range of the x-axis e.g. [4, 8].
         comp : Corr or list of Corr
             Correlator or list of correlators which are plotted for comparison.
             The tags of these correlators are used as labels if available.
         logscale : bool
-            Sets y-axis to logscale
+            Sets y-axis to logscale.
         plateau : Obs
-            Plateau value to be visualized in the figure
+            Plateau value to be visualized in the figure.
         fit_res : Fit_result
-            Fit_result object to be visualized
+            Fit_result object to be visualized.
         ylabel : str
-            Label for the y-axis
+            Label for the y-axis.
         save : str
-            path to file in which the figure should be saved
+            path to file in which the figure should be saved.
         auto_gamma : bool
             Apply the gamma method with standard parameters to all correlators and plateau values before plotting.
         hide_sigma : float
             Hides data points from the first value on which is consistent with zero within 'hide_sigma' standard errors.
         references : list
             List of floating point values that are displayed as horizontal lines for reference.
+        title : string
+            Optional title of the figure.
         """
         if self.N != 1:
             raise Exception("Correlator must be projected before plotting")
@@ -798,7 +825,7 @@ class Corr:
                         hide_from = np.argmax((hide_sigma * np.array(y_err[1:])) > np.abs(y[1:])) - 1
                     else:
                         hide_from = None
-                    plt.errorbar(x[:hide_from], y[:hide_from], y_err[:hide_from], label=corr.tag, mfc=plt.rcParams['axes.facecolor'])
+                    ax1.errorbar(x[:hide_from], y[:hide_from], y_err[:hide_from], label=corr.tag, mfc=plt.rcParams['axes.facecolor'])
             else:
                 raise Exception("'comp' must be a correlator or a list of correlators.")
 
@@ -836,11 +863,15 @@ class Corr:
         handles, labels = ax1.get_legend_handles_labels()
         if labels:
             ax1.legend()
+
+        if title:
+            plt.title(title)
+
         plt.draw()
 
         if save:
             if isinstance(save, str):
-                fig.savefig(save)
+                fig.savefig(save, bbox_inches='tight')
             else:
                 raise Exception("'save' has to be a string.")
 
@@ -897,12 +928,14 @@ class Corr:
         else:
             raise Exception("Unknown datatype " + str(datatype))
 
-    def print(self, range=[0, None]):
-        print(self.__repr__(range))
+    def print(self, print_range=None):
+        print(self.__repr__(print_range))
+
+    def __repr__(self, print_range=None):
+        if print_range is None:
+            print_range = [0, None]
 
-    def __repr__(self, range=[0, None]):
         content_string = ""
-
         content_string += "Corr T=" + str(self.T) + " N=" + str(self.N) + "\n"  # +" filled with"+ str(type(self.content[0][0])) there should be a good solution here
 
         if self.tag is not None:
@@ -910,14 +943,14 @@ class Corr:
         if self.N != 1:
             return content_string
 
-        if range[1]:
-            range[1] += 1
+        if print_range[1]:
+            print_range[1] += 1
         content_string += 'x0/a\tCorr(x0/a)\n------------------\n'
-        for i, sub_corr in enumerate(self.content[range[0]:range[1]]):
+        for i, sub_corr in enumerate(self.content[print_range[0]:print_range[1]]):
             if sub_corr is None:
-                content_string += str(i + range[0]) + '\n'
+                content_string += str(i + print_range[0]) + '\n'
             else:
-                content_string += str(i + range[0])
+                content_string += str(i + print_range[0])
                 for element in sub_corr:
                     content_string += '\t' + ' ' * int(element >= 0) + str(element)
                 content_string += '\n'
@@ -961,7 +994,7 @@ class Corr:
 
     def __mul__(self, y):
         if isinstance(y, Corr):
-            if not((self.N == 1 or y.N == 1 or self.N == y.N) and self.T == y.T):
+            if not ((self.N == 1 or y.N == 1 or self.N == y.N) and self.T == y.T):
                 raise Exception("Multiplication of Corr object requires N=N or N=1 and T=T")
             newcontent = []
             for t in range(self.T):
@@ -989,7 +1022,7 @@ class Corr:
 
     def __truediv__(self, y):
         if isinstance(y, Corr):
-            if not((self.N == 1 or y.N == 1 or self.N == y.N) and self.T == y.T):
+            if not ((self.N == 1 or y.N == 1 or self.N == y.N) and self.T == y.T):
                 raise Exception("Multiplication of Corr object requires N=N or N=1 and T=T")
             newcontent = []
             for t in range(self.T):
