@@ -1,4 +1,5 @@
 import warnings
+import hashlib
 import pickle
 from math import gcd
 from functools import reduce
@@ -266,33 +267,45 @@ class Obs:
                 self.e_windowsize[e_name] = 0
                 continue
 
+            gaps = []
+            for r_name in e_content[e_name]:
+                if isinstance(self.idl[r_name], range):
+                    gaps.append(1)
+                else:
+                    gaps.append(np.min(np.diff(self.idl[r_name])))
+
+            if not np.all([gi == gaps[0] for gi in gaps]):
+                raise Exception(f"Replica for ensemble {e_name} are not equally spaced.", gaps)
+            else:
+                gapsize = gaps[0]
+
             self.e_rho[e_name] = e_gamma[e_name][:w_max] / e_gamma[e_name][0]
             self.e_n_tauint[e_name] = np.cumsum(np.concatenate(([0.5], self.e_rho[e_name][1:])))
             # Make sure no entry of tauint is smaller than 0.5
             self.e_n_tauint[e_name][self.e_n_tauint[e_name] <= 0.5] = 0.5 + np.finfo(np.float64).eps
             # hep-lat/0306017 eq. (42)
-            self.e_n_dtauint[e_name] = self.e_n_tauint[e_name] * 2 * np.sqrt(np.abs(np.arange(w_max) + 0.5 - self.e_n_tauint[e_name]) / e_N)
+            self.e_n_dtauint[e_name] = self.e_n_tauint[e_name] * 2 * np.sqrt(np.abs(np.arange(w_max) / gapsize + 0.5 - self.e_n_tauint[e_name]) / e_N)
             self.e_n_dtauint[e_name][0] = 0.0
 
             def _compute_drho(i):
                 tmp = self.e_rho[e_name][i + 1:w_max] + np.concatenate([self.e_rho[e_name][i - 1::-1], self.e_rho[e_name][1:w_max - 2 * i]]) - 2 * self.e_rho[e_name][i] * self.e_rho[e_name][1:w_max - i]
                 self.e_drho[e_name][i] = np.sqrt(np.sum(tmp ** 2) / e_N)
 
-            _compute_drho(1)
+            _compute_drho(gapsize)
             if self.tau_exp[e_name] > 0:
                 texp = self.tau_exp[e_name]
                 # Critical slowing down analysis
                 if w_max // 2 <= 1:
                     raise Exception("Need at least 8 samples for tau_exp error analysis")
-                for n in range(1, w_max // 2):
-                    _compute_drho(n + 1)
+                for n in range(gapsize, w_max // 2, gapsize):
+                    _compute_drho(n + gapsize)
                     if (self.e_rho[e_name][n] - self.N_sigma[e_name] * self.e_drho[e_name][n]) < 0 or n >= w_max // 2 - 2:
                         # Bias correction hep-lat/0306017 eq. (49) included
-                        self.e_tauint[e_name] = self.e_n_tauint[e_name][n] * (1 + (2 * n + 1) / e_N) / (1 + 1 / e_N) + texp * np.abs(self.e_rho[e_name][n + 1])  # The absolute makes sure, that the tail contribution is always positive
+                        self.e_tauint[e_name] = self.e_n_tauint[e_name][n] * (1 + (2 * n / gapsize + 1) / e_N) / (1 + 1 / e_N) + texp * np.abs(self.e_rho[e_name][n + 1])  # The absolute makes sure, that the tail contribution is always positive
                         self.e_dtauint[e_name] = np.sqrt(self.e_n_dtauint[e_name][n] ** 2 + texp ** 2 * self.e_drho[e_name][n + 1] ** 2)
                         # Error of tau_exp neglected so far, missing term: self.e_rho[e_name][n + 1] ** 2 * d_tau_exp ** 2
                         self.e_dvalue[e_name] = np.sqrt(2 * self.e_tauint[e_name] * e_gamma[e_name][0] * (1 + 1 / e_N) / e_N)
-                        self.e_ddvalue[e_name] = self.e_dvalue[e_name] * np.sqrt((n + 0.5) / e_N)
+                        self.e_ddvalue[e_name] = self.e_dvalue[e_name] * np.sqrt((n / gapsize + 0.5) / e_N)
                         self.e_windowsize[e_name] = n
                         break
             else:
@@ -304,16 +317,17 @@ class Obs:
                     self.e_windowsize[e_name] = 0
                 else:
                     # Standard automatic windowing procedure
-                    tau = self.S[e_name] / np.log((2 * self.e_n_tauint[e_name][1:] + 1) / (2 * self.e_n_tauint[e_name][1:] - 1))
-                    g_w = np.exp(- np.arange(1, w_max) / tau) - tau / np.sqrt(np.arange(1, w_max) * e_N)
+                    tau = self.S[e_name] / np.log((2 * self.e_n_tauint[e_name][gapsize::gapsize] + 1) / (2 * self.e_n_tauint[e_name][gapsize::gapsize] - 1))
+                    g_w = np.exp(- np.arange(1, len(tau) + 1) / tau) - tau / np.sqrt(np.arange(1, len(tau) + 1) * e_N)
                     for n in range(1, w_max):
                         if n < w_max // 2 - 2:
-                            _compute_drho(n + 1)
+                            _compute_drho(gapsize * n + gapsize)
                         if g_w[n - 1] < 0 or n >= w_max - 1:
-                            self.e_tauint[e_name] = self.e_n_tauint[e_name][n] * (1 + (2 * n + 1) / e_N) / (1 + 1 / e_N)  # Bias correction hep-lat/0306017 eq. (49)
+                            n *= gapsize
+                            self.e_tauint[e_name] = self.e_n_tauint[e_name][n] * (1 + (2 * n / gapsize + 1) / e_N) / (1 + 1 / e_N)  # Bias correction hep-lat/0306017 eq. (49)
                             self.e_dtauint[e_name] = self.e_n_dtauint[e_name][n]
                             self.e_dvalue[e_name] = np.sqrt(2 * self.e_tauint[e_name] * e_gamma[e_name][0] * (1 + 1 / e_N) / e_N)
-                            self.e_ddvalue[e_name] = self.e_dvalue[e_name] * np.sqrt((n + 0.5) / e_N)
+                            self.e_ddvalue[e_name] = self.e_dvalue[e_name] * np.sqrt((n / gapsize + 0.5) / e_N)
                             self.e_windowsize[e_name] = n
                             break
 
@@ -385,13 +399,24 @@ class Obs:
             print('Result\t %3.8e +/- %3.8e +/- %3.8e (%3.3f%%)' % (self.value, self._dvalue, self.ddvalue, percentage))
             if len(self.e_names) > 1:
                 print(' Ensemble errors:')
+            e_content = self.e_content
             for e_name in self.mc_names:
-                if len(self.e_names) > 1:
-                    print('', e_name, '\t %3.8e +/- %3.8e' % (self.e_dvalue[e_name], self.e_ddvalue[e_name]))
-                if self.tau_exp[e_name] > 0:
-                    print(' t_int\t %3.8e +/- %3.8e tau_exp = %3.2f,  N_sigma = %1.0i' % (self.e_tauint[e_name], self.e_dtauint[e_name], self.tau_exp[e_name], self.N_sigma[e_name]))
+                if isinstance(self.idl[e_content[e_name][0]], range):
+                    gap = self.idl[e_content[e_name][0]].step
                 else:
-                    print(' t_int\t %3.8e +/- %3.8e S = %3.2f' % (self.e_tauint[e_name], self.e_dtauint[e_name], self.S[e_name]))
+                    gap = np.min(np.diff(self.idl[e_content[e_name][0]]))
+
+                if len(self.e_names) > 1:
+                    print('', e_name, '\t %3.6e +/- %3.6e' % (self.e_dvalue[e_name], self.e_ddvalue[e_name]))
+                tau_string = " \N{GREEK SMALL LETTER TAU}_int\t " + _format_uncertainty(self.e_tauint[e_name], self.e_dtauint[e_name])
+                tau_string += f" in units of {gap} config"
+                if gap > 1:
+                    tau_string += "s"
+                if self.tau_exp[e_name] > 0:
+                    tau_string = f"{tau_string: <45}" + '\t(\N{GREEK SMALL LETTER TAU}_exp=%3.2f, N_\N{GREEK SMALL LETTER SIGMA}=%1.0i)' % (self.tau_exp[e_name], self.N_sigma[e_name])
+                else:
+                    tau_string = f"{tau_string: <45}" + '\t(S=%3.2f)' % (self.S[e_name])
+                print(tau_string)
             for e_name in self.cov_names:
                 print('', e_name, '\t %3.8e' % (self.e_dvalue[e_name]))
         if ens_content is True:
@@ -408,7 +433,7 @@ class Obs:
                         if isinstance(self.idl[value[0]], range):
                             my_string += f' (from {self.idl[value[0]].start} to {self.idl[value[0]][-1]}' + int(self.idl[value[0]].step != 1) * f' in steps of {self.idl[value[0]].step}' + ')'
                         else:
-                            my_string += ' (irregular range)'
+                            my_string += f' (irregular range from {self.idl[value[0]][0]} to {self.idl[value[0]][-1]})'
                     else:
                         sublist = []
                         for v in value:
@@ -417,7 +442,7 @@ class Obs:
                             if isinstance(self.idl[v], range):
                                 my_substring += f' (from {self.idl[v].start} to {self.idl[v][-1]}' + int(self.idl[v].step != 1) * f' in steps of {self.idl[v].step}' + ')'
                             else:
-                                my_substring += ' (irregular range)'
+                                my_substring += f' (irregular range from {self.idl[v][0]} to {self.idl[v][-1]})'
                             sublist.append(my_substring)
 
                         my_string += '\n' + '\n'.join(sublist)
@@ -426,6 +451,21 @@ class Obs:
                 my_string_list.append(my_string)
             print('\n'.join(my_string_list))
 
+    def reweight(self, weight):
+        """Reweight the obs with given rewighting factors.
+
+        Parameters
+        ----------
+        weight : Obs
+            Reweighting factor. An Observable that has to be defined on a superset of the
+            configurations in obs[i].idl for all i.
+        all_configs : bool
+            if True, the reweighted observables are normalized by the average of
+            the reweighting factor on all configurations in weight.idl and not
+            on the configurations in obs[i].idl. Default False.
+        """
+        return reweight(weight, [self])[0]
+
     def is_zero_within_error(self, sigma=1):
         """Checks whether the observable is zero within 'sigma' standard errors.
 
@@ -661,13 +701,16 @@ class Obs:
     def __str__(self):
         if self._dvalue == 0.0:
             return str(self.value)
-        fexp = np.floor(np.log10(self._dvalue))
-        if fexp < 0.0:
-            return '{:{form}}({:2.0f})'.format(self.value, self._dvalue * 10 ** (-fexp + 1), form='.' + str(-int(fexp) + 1) + 'f')
-        elif fexp == 0.0:
-            return '{:.1f}({:1.1f})'.format(self.value, self._dvalue)
-        else:
-            return '{:.0f}({:2.0f})'.format(self.value, self._dvalue)
+        return _format_uncertainty(self.value, self._dvalue)
+
+    def __hash__(self):
+        hash_tuple = (np.array([self.value]).astype(np.float32).data.tobytes(),)
+        hash_tuple += tuple([o.astype(np.float32).data.tobytes() for o in self.deltas.values()])
+        hash_tuple += tuple([np.array([o.errsq()]).astype(np.float32).data.tobytes() for o in self.covobs.values()])
+        hash_tuple += tuple([o.encode() for o in self.names])
+        m = hashlib.md5()
+        [m.update(o) for o in hash_tuple]
+        return int(m.hexdigest(), 16) & 0xFFFFFFFF
 
     # Overload comparisons
     def __lt__(self, other):
@@ -934,6 +977,17 @@ class CObs:
         return 'CObs[' + str(self) + ']'
 
 
+def _format_uncertainty(value, dvalue):
+    """Creates a string of a value and its error in paranthesis notation, e.g., 13.02(45)"""
+    fexp = np.floor(np.log10(dvalue))
+    if fexp < 0.0:
+        return '{:{form}}({:2.0f})'.format(value, dvalue * 10 ** (-fexp + 1), form='.' + str(-int(fexp) + 1) + 'f')
+    elif fexp == 0.0:
+        return '{:.1f}({:1.1f})'.format(value, dvalue)
+    else:
+        return '{:.0f}({:2.0f})'.format(value, dvalue)
+
+
 def _expand_deltas(deltas, idx, shape):
     """Expand deltas defined on idx to a regular, contiguous range, where holes are filled by 0.
        If idx is of type range, the deltas are not changed
@@ -1330,7 +1384,7 @@ def reweight(weight, obs, **kwargs):
     all_configs : bool
         if True, the reweighted observables are normalized by the average of
         the reweighting factor on all configurations in weight.idl and not
-        on the configurations in obs[i].idl.
+        on the configurations in obs[i].idl. Default False.
     """
     result = []
     for i in range(len(obs)):
@@ -1353,7 +1407,7 @@ def reweight(weight, obs, **kwargs):
         else:
             new_weight = Obs([w_deltas[name] + weight.r_values[name] for name in sorted(obs[i].names)], sorted(obs[i].names), idl=[obs[i].idl[name] for name in sorted(obs[i].names)])
 
-        result.append(derived_observable(lambda x, **kwargs: x[0] / x[1], [tmp_obs, new_weight], **kwargs))
+        result.append(tmp_obs / new_weight)
         result[-1].reweighted = True
         result[-1].is_merged = obs[i].is_merged
 
